﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace TaskAndAwait
{
    public partial class MainForm : Form
    {
        PersonRepository repository;
        CancellationTokenSource tokenSource;
        CancellationToken token;

        public MainForm()
        {
            InitializeComponent();
            repository = new PersonRepository();
            ResetTokenSource();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }

        private async void buttonAwait_Click(object sender, EventArgs e)
        {
            Task<List<Person>> personTask = null;
            try
            {
                ClearListBox();
                ResetTokenSource();
                personTask = repository.Get(token);
                if (checkBoxCancel.Checked)
                {
                    Thread.Sleep(1000);
                    tokenSource.Cancel();
                }
                List<Person> people = await personTask;
                foreach (Person person in people)
                {
                    listBoxPerson.Items.Add(person);
                }
            }
            catch (OperationCanceledException)
            {
                ProcessCancel();
            }
            catch (Exception ex)
            {
                if (personTask.IsFaulted)
                {
                    ProcessException(personTask.Exception);
                }
                else
                {
                    MessageBox.Show(ex.Message, "Error UI");
                }
            }
        }

        void FillListBox(Task<List<Person>> peopleTask)
        {
            List<Person> people = peopleTask.Result;
            foreach (Person person in people) {
                listBoxPerson.Items.Add(person);
            }
        }

        void ProcessCancel()
        {
            MessageBox.Show("Operation Cancelled", "Cancelled");
        }

        void ProcessException(AggregateException aggregate)
        {
            foreach (var ex in aggregate.Flatten().InnerExceptions)
            {
                MessageBox.Show(ex.Message, "Error");
            }
        }

        void ProcessResult(List<Person> people)
        {
            foreach (Person person in people)
            {
                listBoxPerson.Items.Add(person);
            }
        }

        void FillListBoxStatus(Task<List<Person>> peopleTask)
        {
            switch (peopleTask.Status)
            {
                case TaskStatus.Canceled:
                    ProcessCancel();
                    break;
                case TaskStatus.Faulted:
                    ProcessException(peopleTask.Exception);
                    break;
                case TaskStatus.RanToCompletion:
                    ProcessResult(peopleTask.Result);
                    break;
                default:
                    break;
            }
        }

        private void ResetTokenSource()
        {
            if (tokenSource != null)
            {
                tokenSource.Dispose();
            }
            tokenSource = new CancellationTokenSource();
            token = tokenSource.Token;
        }

        private void buttonTask_Click(object sender, EventArgs e)
        {
            ClearListBox();
            ResetTokenSource();
            Task<List<Person>> peopleTask = repository.Get(token);
            peopleTask.ContinueWith(FillListBoxStatus,
                TaskScheduler.FromCurrentSynchronizationContext());
            if (checkBoxCancel.Checked) { 
                Thread.Sleep(1000);
                tokenSource.Cancel();
            }
        }

        private void buttonClear_Click(object sender, EventArgs e)
        {
            ClearListBox();
        }

        private void ClearListBox()
        {
            listBoxPerson.Items.Clear();
        }
    }
}
